@@ -3,10 +3,14 @@ package version;
 
 use 5.006002;
 use strict;
+use warnings::register;
+if ($] >= 5.015) {
+    warnings::register_categories(qw/version/);
+}
 
 use vars qw(@ISA $VERSION $CLASS $STRICT $LAX *declare *qv);
 
-$VERSION = 0.9909;
+$VERSION = 0.9910;
 $CLASS = 'version';
 
 # !!!!Delete this next block completely when adding to Perl core!!!!
@@ -23,6 +27,8 @@ $CLASS = 'version';
 	*version::_VERSION = \&version::vpp::_VERSION;
 	*version::vcmp = \&version::vpp::vcmp;
 	*version::new = \&version::vpp::new;
+	*version::numify = \&version::vpp::numify;
+	*version::normal = \&version::vpp::normal;
 	if ($] >= 5.009000) {
 	    no strict 'refs';
 	    *version::stringify = \&version::vpp::stringify;
@@ -39,6 +45,8 @@ $CLASS = 'version';
 	*version::_VERSION = \&version::vxs::_VERSION;
 	*version::vcmp = \&version::vxs::VCMP;
 	*version::new = \&version::vxs::new;
+	*version::numify = \&version::vxs::numify;
+	*version::normal = \&version::vxs::normal;
 	if ($] >= 5.009000) {
 	    no strict 'refs';
 	    *version::stringify = \&version::vxs::stringify;
