@@ -1,8 +1,6 @@
 #! /usr/local/perl -w
 package main;
 require Test::Harness;
-*Verbose = \$Test::Harness::Verbose;
-*Verbose = 0 if $ENV{PERL_CORE};
 use Data::Dumper;
 use File::Temp qw/tempfile/;
 use File::Basename;
@@ -27,21 +25,22 @@ sub BaseTests {
     # its man page ( perldoc Test ) for help writing this test script.
 
     # Test bare number processing
-    diag "tests with bare numbers" if $Verbose;
     $version = $CLASS->$method(5.005_03);
     is ( "$version" , "5.00503" , '5.005_03 eq 5.00503' );
     $version = $CLASS->$method(1.23);
     is ( "$version" , "1.23" , '1.23 eq "1.23"' );
 
+    # Test explicit integer
+    $version = $CLASS->$method(23);
+    is ( "$version" , 23 , '23 eq "23"' );
+
     # Test quoted number processing
-    diag "tests with quoted numbers" if $Verbose;
     $version = $CLASS->$method("5.005_03");
     is ( "$version" , "5.005_03" , '"5.005_03" eq "5.005_03"' );
     $version = $CLASS->$method("v1.23");
     is ( "$version" , "v1.23" , '"v1.23" eq "v1.23"' );
 
     # Test stringify operator
-    diag "tests with stringify" if $Verbose;
     $version = $CLASS->$method("5.005");
     is ( "$version" , "5.005" , '5.005 eq "5.005"' );
     $version = $CLASS->$method("5.006.001");
@@ -51,7 +50,6 @@ sub BaseTests {
     is ( "$version" , "v1.2.3_4" , 'alpha version 1.2.3_4 eq v1.2.3_4' );
 
     # test illegal formats
-    diag "test illegal formats" if $Verbose;
     eval {my $version = $CLASS->$method("1.2_3_4")};
     like($@, qr/multiple underscores/,
 	"Invalid version format (multiple underscores)");
@@ -93,7 +91,6 @@ sub BaseTests {
     isa_ok ( $version, $CLASS );
 
     # Test comparison operators with self
-    diag "tests with self" if $Verbose;
     is ( $version <=> $version, 0, '$version <=> $version == 0' );
     ok ( $version == $version, '$version == $version' );
 
@@ -101,7 +98,6 @@ sub BaseTests {
     # test first with non-object
     $version = $CLASS->$method("5.006.001");
     $new_version = "5.8.0";
-    diag "numeric tests with non-objects" if $Verbose;
     ok ( $version == $version, '$version == $version' );
     ok ( $version < $new_version, '$version < $new_version' );
     ok ( $new_version > $version, '$new_version > $version' );
@@ -109,20 +105,17 @@ sub BaseTests {
 
     # now test with existing object
     $new_version = $CLASS->$method($new_version);
-    diag "numeric tests with objects" if $Verbose;
     ok ( $version < $new_version, '$version < $new_version' );
     ok ( $new_version > $version, '$new_version > $version' );
     ok ( $version != $new_version, '$version != $new_version' );
 
     # now test with actual numbers
-    diag "numeric tests with numbers" if $Verbose;
     ok ( $version->numify() == 5.006001, '$version->numify() == 5.006001' );
     ok ( $version->numify() <= 5.006001, '$version->numify() <= 5.006001' );
     ok ( $version->numify() < 5.008, '$version->numify() < 5.008' );
     #ok ( $version->numify() > v5.005_02, '$version->numify() > 5.005_02' );
 
     # test with long decimals
-    diag "Tests with extended decimal versions" if $Verbose;
     $version = $CLASS->$method(1.002003);
     ok ( $version == "1.2.3", '$version == "1.2.3"');
     ok ( $version->numify == 1.002003, '$version->numify == 1.002003');
@@ -134,14 +127,11 @@ sub BaseTests {
     # now test with alpha version form with string
     $version = $CLASS->$method("1.2.3");
     $new_version = "1.2.3_4";
-    diag "numeric tests with alpha-style non-objects" if $Verbose;
     ok ( $version < $new_version, '$version < $new_version' );
     ok ( $new_version > $version, '$new_version > $version' );
     ok ( $version != $new_version, '$version != $new_version' );
 
     $version = $CLASS->$method("1.2.4");
-    diag "numeric tests with alpha-style non-objects"
-	if $Verbose;
     ok ( $version > $new_version, '$version > $new_version' );
     ok ( $new_version < $version, '$new_version < $version' );
     ok ( $version != $new_version, '$version != $new_version' );
@@ -149,7 +139,6 @@ sub BaseTests {
     # now test with alpha version form with object
     $version = $CLASS->$method("1.2.3");
     $new_version = $CLASS->$method("1.2.3_4");
-    diag "tests with alpha-style objects" if $Verbose;
     ok ( $version < $new_version, '$version < $new_version' );
     ok ( $new_version > $version, '$new_version > $version' );
     ok ( $version != $new_version, '$version != $new_version' );
@@ -157,20 +146,16 @@ sub BaseTests {
     ok ( $new_version->is_alpha, '$new_version->is_alpha');
 
     $version = $CLASS->$method("1.2.4");
-    diag "tests with alpha-style objects" if $Verbose;
     ok ( $version > $new_version, '$version > $new_version' );
     ok ( $new_version < $version, '$new_version < $version' );
     ok ( $version != $new_version, '$version != $new_version' );
 
     $version = $CLASS->$method("1.2.3.4");
     $new_version = $CLASS->$method("1.2.3_4");
-    diag "tests with alpha-style objects with same subversion"
-	if $Verbose;
     ok ( $version > $new_version, '$version > $new_version' );
     ok ( $new_version < $version, '$new_version < $version' );
     ok ( $version != $new_version, '$version != $new_version' );
 
-    diag "test implicit [in]equality" if $Verbose;
     $version = $CLASS->$method("v1.2.3");
     $new_version = $CLASS->$method("1.2.3.0");
     ok ( $version == $new_version, '$version == $new_version' );
@@ -183,7 +168,6 @@ sub BaseTests {
     $new_version = $CLASS->$method("1.1.999");
     ok ( $version > $new_version, '$version > $new_version' );
 
-    diag "test with version class names" if $Verbose;
     $version = $CLASS->$method("v1.2.3");
     eval { () = $version < 'version' };
     # this test, and only this test, I have to do this or else $@ gets
@@ -192,7 +176,6 @@ sub BaseTests {
     like $err, qr/^Invalid version format/, "error with $version < 'version'";
 
     # that which is not expressly permitted is forbidden
-    diag "forbidden operations" if $Verbose;
     ok ( !eval { ++$version }, "noop ++" );
     ok ( !eval { --$version }, "noop --" );
     ok ( !eval { $version/1 }, "noop /" );
@@ -203,7 +186,6 @@ SKIP: {
     skip "version require'd instead of use'd, cannot test $qv_declare", 3
 	unless defined $qv_declare;
     # test the $qv_declare() sub
-    diag "testing $qv_declare" if $Verbose;
     $version = $CLASS->$qv_declare("1.2");
     is ( "$version", "v1.2", $qv_declare.'("1.2") == "1.2.0"' );
     $version = $CLASS->$qv_declare(1.2);
@@ -212,7 +194,6 @@ SKIP: {
 }
 
     # test creation from existing version object
-    diag "create new from existing version" if $Verbose;
     ok (eval {$new_version = $CLASS->$method($version)},
 	    "new from existing object");
     ok ($new_version == $version, "class->$method($version) identical");
@@ -223,21 +204,18 @@ SKIP: {
     is ($new_version, "1.2.3" , '$version->$method("1.2.3") works too');
 
     # test the CVS revision mode
-    diag "testing CVS Revision" if $Verbose;
     $version = new $CLASS qw$Revision: 1.2$;
     ok ( $version == "1.2.0", 'qw$Revision: 1.2$ == 1.2.0' );
     $version = new $CLASS qw$Revision: 1.2.3.4$;
     ok ( $version == "1.2.3.4", 'qw$Revision: 1.2.3.4$ == 1.2.3.4' );
 
     # test the CPAN style reduced significant digit form
-    diag "testing CPAN-style versions" if $Verbose;
     $version = $CLASS->$method("1.23_01");
     is ( "$version" , "1.23_01", "CPAN-style alpha version" );
     ok ( $version > 1.23, "1.23_01 > 1.23");
     ok ( $version < 1.24, "1.23_01 < 1.24");
 
     # test reformed UNIVERSAL::VERSION
-    diag "Replacement UNIVERSAL::VERSION tests" if $Verbose;
 
     my $error_regex = $] < 5.006
 	? 'version \d required'
@@ -355,7 +333,6 @@ SKIP:    { # https://rt.perl.org/rt3/Ticket/Display.html?id=95544
 SKIP: 	{
 	skip 'Cannot test bare v-strings with Perl < 5.6.0', 4
 		if $] < 5.006_000;
-	diag "Tests with v-strings" if $Verbose;
 	$version = $CLASS->$method(1.2.3);
 	ok("$version" eq "v1.2.3", '"$version" eq 1.2.3');
 	$version = $CLASS->$method(1.0.0);
@@ -370,15 +347,12 @@ SKIP: 	{
 SKIP: 	{
 	skip 'Cannot test bare alpha v-strings with Perl < 5.8.1', 2
 		if $] lt 5.008_001;
-	diag "Tests with bare alpha v-strings" if $Verbose;
 	$version = $CLASS->$method(v1.2.3_4);
 	is($version, "v1.2.3_4", '"$version" eq "v1.2.3_4"');
 	$version = $CLASS->$method(eval "v1.2.3_4");
 	is($version, "v1.2.3_4", '"$version" eq "v1.2.3_4" (from eval)');
     }
 
-    diag "Tests with real-world (malformed) data" if $Verbose;
-
     # trailing zero testing (reported by Andreas Koenig).
     $version = $CLASS->$method("1");
     ok($version->numify eq "1.000", "trailing zeros preserved");
@@ -456,13 +430,13 @@ EOF
     }
 
 SKIP: {
-    skip 'Cannot test "use parent qw(version)"  when require is used', 3
+    skip "Cannot test \"use parent $CLASS\"  when require is used", 3
 	unless defined $qv_declare;
     my ($fh, $filename) = tempfile('tXXXXXXX', SUFFIX => '.pm', UNLINK => 1);
     (my $package = basename($filename)) =~ s/\.pm$//;
     print $fh <<"EOF";
 package $package;
-use parent qw(version);
+use parent $CLASS;
 1;
 EOF
     close $fh;
@@ -520,9 +494,9 @@ EOF
 
     {
 	# http://rt.perl.org/rt3/Ticket/Display.html?id=56606
-	my $badv = bless { version => [1,2,3] }, "version";
+	my $badv = bless { version => [1,2,3] }, $CLASS;
 	is $badv, '1.002003', "Deal with badly serialized versions from YAML";
-	my $badv2 = bless { qv => 1, version => [1,2,3] }, "version";
+	my $badv2 = bless { qv => 1, version => [1,2,3] }, $CLASS;
 	is $badv2, 'v1.2.3', "Deal with badly serialized versions from YAML ";
     }
 
@@ -606,6 +580,61 @@ SKIP: {
 	is $v->numify, '0.520', 'Correctly nummified';
     }
 
+    { # https://rt.cpan.org/Ticket/Display.html?id=88495
+	@ver::ISA = $CLASS;
+	is ref(ver->new), 'ver', 'ver can inherit from version';
+	is ref(ver->qv("1.2.3")), 'ver', 'ver can inherit from version';
+    }
+
+    { # discovered while integrating with bleadperl
+	eval {my $v = $CLASS->new([1,2,3]) };
+	like $@, qr/Invalid version format/, 'Do not crash for garbage';
+	eval {my $v = $CLASS->new({1 => 2}) };
+	like $@, qr/Invalid version format/, 'Do not crash for garbage';
+    }
+    { # https://rt.cpan.org/Ticket/Display.html?id=93340
+	my $v = $CLASS->$method(q[2.6_01]);
+	is $v->normal(), 'v2.601.0', 'Normalized alphas are lossy';
+	like $warning, qr/alpha->normal\(\) is lossy/, 'Heed my warning';
+
+	is $v->numify(), '2.601', 'Numified alphas are lossy';
+	like $warning, qr/alpha->numify\(\) is lossy/, 'Heed my warning';
+
+	$v = $CLASS->$method(q[v2.6_01]);
+	is $v->normal(), 'v2.6.1', 'Normalized alphas are lossy';
+	like $warning, qr/alpha->normal\(\) is lossy/, 'Heed my warning';
+
+	is $v->numify(), '2.006_001', 'Numified alphas are lossy';
+	like $warning, qr/alpha->numify\(\) is lossy/, 'Heed my warning';
+    }
+    { # https://rt.cpan.org/Ticket/Display.html?id=93603
+	eval {my $v = $CLASS->$method('.1.')};
+	like $@, qr/trailing decimal/, 'Forbid trailing decimals';
+	eval {my $v = $CLASS->$method('.1.2.')};
+	like $@, qr/trailing decimal/, 'Forbid trailing decimals';
+    }
+    { # https://rt.cpan.org/Ticket/Display.html?id=93715
+	eval {my $v = $CLASS->new(v1.2)};
+	unlike $@, qr/non-numeric data/, 'Handle short v-strings';
+	eval {my $v = $CLASS->new(v1)};
+	unlike $@, qr/non-numeric data/, 'Handle short v-strings';
+    }
+    {
+	my $two31 = '2147483648';
+	my $v = $CLASS->new($two31);
+	is "$v", 'v.Inf', 'Element Exceeds VERSION_MAX';
+	like $warning, qr/Integer overflow in version/, 'Overflow warning';
+	$v = $CLASS->new("1.$two31.$two31");
+	is "$v", 'v.Inf', 'Element Exceeds VERSION_MAX';
+	like $warning, qr/Integer overflow in version/, 'Overflow warning';
+    }
+    {
+	# now as a number
+	$two31 = 2**31;
+	$v = $CLASS->new($two31);
+	is "$v", 'v.Inf', 'Element Exceeds VERSION_MAX';
+	like $warning, qr/Integer overflow in version/, 'Overflow warning';
+    }
 }
 
 1;
