@@ -119,11 +119,18 @@ package version::vpp;
 
 use 5.006002;
 use strict;
+use warnings::register;
 
 use Config;
-use vars qw($VERSION $CLASS @ISA $LAX $STRICT);
-$VERSION = 0.9907;
+use vars qw($VERSION $CLASS @ISA $LAX $STRICT $WARN_CATEGORY);
+$VERSION = 0.9910;
 $CLASS = 'version::vpp';
+if ($] > 5.015) {
+    warnings::register_categories(qw/version/);
+    $WARN_CATEGORY = 'version';
+} else {
+    $WARN_CATEGORY = 'numeric';
+}
 
 require version::regex;
 *version::vpp::is_strict = \&version::regex::is_strict;
@@ -148,16 +155,6 @@ use overload (
     'abs'      => \&vnoop,
 );
 
-eval "use warnings";
-if ($@) {
-    eval '
-	package
-	warnings;
-	sub enabled {return $^W;}
-	1;
-    ';
-}
-
 sub import {
     no strict 'refs';
     my ($class) = shift;
@@ -196,7 +193,7 @@ sub import {
     }
 
     if (exists($args{'UNIVERSAL::VERSION'})) {
-	local $^W;
+	no warnings qw/redefine/;
 	*UNIVERSAL::VERSION
 		= \&{$CLASS.'::_VERSION'};
     }
@@ -438,6 +435,10 @@ version_prescan_finish:
 	# trailing non-numeric data
 	return BADVERSION($s,$errstr,"Invalid version format (non-numeric data)");
     }
+    if ($saw_decimal > 1 && ($d-1) eq '.') {
+	# no trailing period allowed
+	return BADVERSION($s,$errstr,"Invalid version format (trailing decimal)");
+    }
 
     if (defined $sqv) {
 	$$sqv = $qv;
@@ -634,78 +635,78 @@ sub scan_version {
     return $s;
 }
 
-sub new
-{
-	my ($class, $value) = @_;
-	unless (defined $class) {
-	    require Carp;
-	    Carp::croak('Usage: version::new(class, version)');
-	}
-	my $self = bless ({}, ref ($class) || $class);
-	my $qv = FALSE;
+sub new {
+    my $class = shift;
+    unless (defined $class or $#_ > 1) {
+	require Carp;
+	Carp::croak('Usage: version::new(class, version)');
+    }
 
-	if ( ref($value) && eval('$value->isa("version")') ) {
-	    # Can copy the elements directly
-	    $self->{version} = [ @{$value->{version} } ];
-	    $self->{qv} = 1 if $value->{qv};
-	    $self->{alpha} = 1 if $value->{alpha};
-	    $self->{original} = ''.$value->{original};
-	    return $self;
-	}
+    my $self = bless ({}, ref ($class) || $class);
+    my $qv = FALSE;
 
-	if ($Config{d_setlocale}) {
-	    use POSIX qw/locale_h/;
-	    use if $Config{d_setlocale}, 'locale';
-	    my $currlocale = setlocale(LC_ALL);
+    if ( $#_ == 1 ) { # must be CVS-style
+	$qv = TRUE;
+    }
+    my $value = pop; # always going to be the last element
 
-	    # if the current locale uses commas for decimal points, we
-	    # just replace commas with decimal places, rather than changing
-	    # locales
-	    if ( localeconv()->{decimal_point} eq ',' ) {
-		$value =~ tr/,/./;
-	    }
-	}
-
-	if ( not defined $value or $value =~ /^undef$/ ) {
-	    # RT #19517 - special case for undef comparison
-	    # or someone forgot to pass a value
-	    push @{$self->{version}}, 0;
-	    $self->{original} = "0";
-	    return ($self);
-	}
-
-	if ( $#_ == 2 ) { # must be CVS-style
-	    $value = $_[2];
-	    $qv = TRUE;
-	}
-
-	if (ref($value) =~ m/ARRAY|HASH/) {
-	    require Carp;
-	    Carp::croak("Invalid version format (non-numeric data)");
-	}
-
-	$value = _un_vstring($value);
-
-	# exponential notation
-	if ( $value =~ /\d+.?\d*e[-+]?\d+/ ) {
-	    $value = sprintf("%.9f",$value);
-	    $value =~ s/(0+)$//; # trim trailing zeros
-	}
-
-	my $s = scan_version($value, \$self, $qv);
-
-	if ($s) { # must be something left over
-	    warn("Version string '%s' contains invalid data; "
-                       ."ignoring: '%s'", $value, $s);
-	}
+    if ( ref($value) && eval('$value->isa("version")') ) {
+	# Can copy the elements directly
+	$self->{version} = [ @{$value->{version} } ];
+	$self->{qv} = 1 if $value->{qv};
+	$self->{alpha} = 1 if $value->{alpha};
+	$self->{original} = ''.$value->{original};
+	return $self;
+    }
 
+    if ( not defined $value or $value =~ /^undef$/ ) {
+	# RT #19517 - special case for undef comparison
+	# or someone forgot to pass a value
+	push @{$self->{version}}, 0;
+	$self->{original} = "0";
 	return ($self);
+    }
+
+
+    if (ref($value) =~ m/ARRAY|HASH/) {
+	require Carp;
+	Carp::croak("Invalid version format (non-numeric data)");
+    }
+
+    $value = _un_vstring($value);
+
+    if ($Config{d_setlocale}) {
+	use POSIX qw/locale_h/;
+	use if $Config{d_setlocale}, 'locale';
+	my $currlocale = setlocale(LC_ALL);
+
+	# if the current locale uses commas for decimal points, we
+	# just replace commas with decimal places, rather than changing
+	# locales
+	if ( localeconv()->{decimal_point} eq ',' ) {
+	    $value =~ tr/,/./;
+	}
+    }
+
+    # exponential notation
+    if ( $value =~ /\d+.?\d*e[-+]?\d+/ ) {
+	$value = sprintf("%.9f",$value);
+	$value =~ s/(0+)$//; # trim trailing zeros
+    }
+
+    my $s = scan_version($value, \$self, $qv);
+
+    if ($s) { # must be something left over
+	warn("Version string '%s' contains invalid data; "
+		   ."ignoring: '%s'", $value, $s);
+    }
+
+    return ($self);
 }
 
 *parse = \&new;
 
-sub numify
-{
+sub numify {
     my ($self) = @_;
     unless (_verify($self)) {
 	require Carp;
@@ -717,6 +718,10 @@ sub numify
     my $digit = $self->{version}[0];
     my $string = sprintf("%d.", $digit );
 
+    if ($alpha and warnings::enabled()) {
+	warnings::warn($WARN_CATEGORY, 'alpha->numify() is lossy');
+    }
+
     for ( my $i = 1 ; $i < $len ; $i++ ) {
 	$digit = $self->{version}[$i];
 	if ( $width < 3 ) {
@@ -745,18 +750,22 @@ sub numify
     return $string;
 }
 
-sub normal
-{
+sub normal {
     my ($self) = @_;
     unless (_verify($self)) {
 	require Carp;
 	Carp::croak("Invalid version object");
     }
     my $alpha = $self->{alpha} || "";
+    my $qv = $self->{qv} || "";
     my $len = $#{$self->{version}};
     my $digit = $self->{version}[0];
     my $string = sprintf("v%d", $digit );
 
+    if ($alpha and warnings::enabled()) {
+	warnings::warn($WARN_CATEGORY, 'alpha->normal() is lossy');
+    }
+
     for ( my $i = 1 ; $i < $len ; $i++ ) {
 	$digit = $self->{version}[$i];
 	$string .= sprintf(".%d", $digit);
@@ -764,12 +773,7 @@ sub normal
 
     if ( $len > 0 ) {
 	$digit = $self->{version}[$len];
-	if ( $alpha ) {
-	    $string .= sprintf("_%0d", $digit);
-	}
-	else {
-	    $string .= sprintf(".%0d", $digit);
-	}
+	$string .= sprintf(".%0d", $digit);
     }
 
     if ( $len <= 2 ) {
@@ -781,8 +785,7 @@ sub normal
     return $string;
 }
 
-sub stringify
-{
+sub stringify {
     my ($self) = @_;
     unless (_verify($self)) {
 	require Carp;
@@ -795,8 +798,7 @@ sub stringify
 	    : $self->numify;
 }
 
-sub vcmp
-{
+sub vcmp {
     require UNIVERSAL;
     my ($left,$right,$swap) = @_;
     my $class = ref($left);
@@ -928,16 +930,16 @@ sub _is_non_alphanumeric {
 sub _un_vstring {
     my $value = shift;
     # may be a v-string
-    if ( length($value) >= 3 && $value !~ /[._]/
+    if ( length($value) >= 1 && $value !~ /[,._]/
 	&& _is_non_alphanumeric($value)) {
 	my $tvalue;
-	if ( $] ge 5.008_001 ) {
+	if ( $] >= 5.008_001 ) {
 	    $tvalue = _find_magic_vstring($value);
 	    $value = $tvalue if length $tvalue;
 	}
-	elsif ( $] ge 5.006_000 ) {
+	elsif ( $] >= 5.006_000 ) {
 	    $tvalue = sprintf("v%vd",$value);
-	    if ( $tvalue =~ /^v\d+(\.\d+){2,}$/ ) {
+	    if ( $tvalue =~ /^v\d+(\.\d+)*$/ ) {
 		# must be a v-string
 		$value = $tvalue;
 	    }
